#
#
# zlister.zbonus.FixFSPerms.awk  Author:   Joe Wulf - 20070309@18:30, Md.
#                                          Copyright (C) 1997-2007, Joe Wulf
#
# "*--------------------------------------------------------------------------------------*"
# "|                                                                                      |"
# "|                             zlister.zbonus.FixFSPerms.awk                            |"
# "|                                                                                      |"
# "|  Companion tool to zlister for comprehensivly managing complete filesystem listings  |"
# "| ------------------------------------------------------------------------------------ |"
# "|                                                                                      |"
# "| These scripts are free software; you can redistribute and/or modify them under the   |"
# "| terms of the GNU General Public License as published by the Free Software            |"
# "| Foundation; either version 2 of the License, or (at your option) any later version.  |"
# "|                                                                                      |"
# "| This program is distributed in the hope that it will be useful, but WITHOUT ANY      |"
# "| WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      |"
# "| PARTICULAR PURPOSE.  See the enclosed GNU General Public License for more details.   |"
# "|                                                                                      |"
# "*----------------------------------------------------[Copyright (C) 2007, Joe Wulf]----*"
#
# "*--------------------------------------------------------------------------------------*"
# "|                                                                                      |"
# "| Please send me your comments, suggestions and constructive critizisms.  All thoughts |"
# "| are welcome.  I WILL give you credit in the appropriate scripts as well as send you  |"
# "| the newest version.  Your efforts will improve this zlister project for everyone.    |"
# "| To contact me, please write me an email at Joe_Wulf@yahoo.com                        |"
# "|                                                                                      |"
# "| Thank you for your time (Joe Wulf)                                                   |"
# "|                                                                                      |"
# "*--------------------------------------------------------------------------------------*"
#
#
# This awk script resides at '/sysadmin/zlister.bin/zlister.zbonus.FixFSPerms.awk' and is
#   executed by '/sysadmin/zlister.bin/zlister.zbonus.FixFSPerms.csh'.  See that script for input.
#
#


BEGIN {
        SD = "/."; SDD = "/.."
      }

NF > 8 {


         # This is a nice-to-have companion script to zlister which will allow an administrator
         #   to put permissions on a system back to the way they were.  Using this script assumes that
         #   zlister has already been executed and that the file containing the filesystem
         #   listing contains the correct permissions, ownership and groupship.
         # Assuming those are true, zcat the filesystem listing to a flat file; then use that
         #   flat filename as the argument to this script.
         #
         # This does its job ONLY for plain-vanilla unix filesystem permissions.  So, TrustedSolaris isn't a supported OS.
         #
         f1 = $1;                # Unparsed permissions field
         x  = substr(f1,1,1)
         if ( x == "l" ) {
                           # This is a link and therefore I won't set permissions on it.
                           #
                           # The point here is that a link, by definition POINTs directly to an entity.  So, when we 'change' a filesystem attribute
                           # of a 'link', then the entity itself IS changed, 'cause the shell won't let the permissions, owner, or group get changed.
                           print "echo '[link Bypassed...]----------------------------------------'"    >> OutFile;
                           print "echo \"" $0 "\""                                                      >> OutFile;
                           print " "                                                                    >> OutFile;
                           next
                         }

         # Logic to 'next'-past any lines ending in "/." and "/..".  Changing their perissions is simply unnecesary, as only the Directory itself
         #  requires the changes.  "/." and "/.." are effectively LINKS back to the current directory and its parent respectively.
         l      = length($0);
         lm2    = l - 1;
         lm3    = l - 2;
         ChkSD  = substr($0,lm2,2);
         ChkSDD = substr($0,lm3,3);
         if ( ChkSD == SD || ChkSDD == SDD ) {
                                               # Lets show that we've passed the respective record.
                                               print "echo '[Record Bypassed...]--------------------------------------'"                     >> OutFile;
                                               print "echo \"" $0 "\""                                                                       >> OutFile;
                                               print "echo \"l(" l "), lm2(" lm2 "), lm3(" lm3 "), ChkSD(" ChkSD "), ChkSDD(" ChkSDD ").\""  >> OutFile;
                                               print " "                                                                                     >> OutFile;
                                               next
                                             };

         O1 = substr(f1,2,3);    # Owner permissions
         G1 = substr(f1,5,3);    # Group permissions
         W1 = substr(f1,8,3);    # World permissions

         # Set Special to a default of zero (0).
         Spcl = 0;

         # Get perms for Owner
         Oxxx = 8
            # Check normal permissions (only 1 could be true).
            # Could add 1 to a variable when a match is made, and unset it when applying
            #   permissions for this entity.  If reached the end of checking permissions
            #   and a match wasn't found, then display an error and keep processing the
            #   next line.
            if ( O1 == "---" ) { Oxxx = 0 };
            if ( O1 == "--x" ) { Oxxx = 1 };
            if ( O1 == "-w-" ) { Oxxx = 2 };
            if ( O1 == "-wx" ) { Oxxx = 3 };
            if ( O1 == "r--" ) { Oxxx = 4 };
            if ( O1 == "r-x" ) { Oxxx = 5 };
            if ( O1 == "rw-" ) { Oxxx = 6 };
            if ( O1 == "rwx" ) { Oxxx = 7 };

            # Check for Set UID permissions (only 1 could be true).
            if ( O1 == "--S" ) { Oxxx = 0; Spcl = Spcl + 4 };
            if ( O1 == "--s" ) { Oxxx = 1; Spcl = Spcl + 4 };
            if ( O1 == "-wS" ) { Oxxx = 2; Spcl = Spcl + 4 };
            if ( O1 == "-ws" ) { Oxxx = 3; Spcl = Spcl + 4 };
            if ( O1 == "r-S" ) { Oxxx = 4; Spcl = Spcl + 4 };
            if ( O1 == "r-s" ) { Oxxx = 5; Spcl = Spcl + 4 };
            if ( O1 == "rwS" ) { Oxxx = 6; Spcl = Spcl + 4 };
            if ( O1 == "rws" ) { Oxxx = 7; Spcl = Spcl + 4 };

         # Get perms for Group
         Gxxx = 8
            # Check normal permissions (only 1 could be true).
            if ( G1 == "---" ) { Gxxx = 0 };
            if ( G1 == "--x" ) { Gxxx = 1 };
            if ( G1 == "-w-" ) { Gxxx = 2 };
            if ( G1 == "-wx" ) { Gxxx = 3 };
            if ( G1 == "r--" ) { Gxxx = 4 };
            if ( G1 == "r-x" ) { Gxxx = 5 };
            if ( G1 == "rw-" ) { Gxxx = 6 };
            if ( G1 == "rwx" ) { Gxxx = 7 };

            # Check for Set GID permissions (only 1 could be true).
            if ( G1 == "--S" ) { Gxxx = 0; Spcl = Spcl + 2 };
            if ( G1 == "--s" ) { Gxxx = 1; Spcl = Spcl + 2 };
            if ( G1 == "-wS" ) { Gxxx = 2; Spcl = Spcl + 2 };
            if ( G1 == "-ws" ) { Gxxx = 3; Spcl = Spcl + 2 };
            if ( G1 == "r-S" ) { Gxxx = 4; Spcl = Spcl + 2 };
            if ( G1 == "r-s" ) { Gxxx = 5; Spcl = Spcl + 2 };
            if ( G1 == "rwS" ) { Gxxx = 6; Spcl = Spcl + 2 };
            if ( G1 == "rws" ) { Gxxx = 7; Spcl = Spcl + 2 };

         # Get perms for World
         Wxxx = 8
            # Check normal permissions (only 1 could be true).
            if ( W1 == "---" ) { Wxxx = 0 };
            if ( W1 == "--x" ) { Wxxx = 1 };
            if ( W1 == "-w-" ) { Wxxx = 2 };
            if ( W1 == "-wx" ) { Wxxx = 3 };
            if ( W1 == "r--" ) { Wxxx = 4 };
            if ( W1 == "r-x" ) { Wxxx = 5 };
            if ( W1 == "rw-" ) { Wxxx = 6 };
            if ( W1 == "rwx" ) { Wxxx = 7 };

            # Check for Sticky bit set (only 1 could be true).
            if ( W1 == "--T" ) { Wxxx = 0; Spcl = Spcl + 1 };
            if ( W1 == "--t" ) { Wxxx = 1; Spcl = Spcl + 1 };
            if ( W1 == "-wT" ) { Wxxx = 2; Spcl = Spcl + 1 };
            if ( W1 == "-wt" ) { Wxxx = 3; Spcl = Spcl + 1 };
            if ( W1 == "r-T" ) { Wxxx = 4; Spcl = Spcl + 1 };
            if ( W1 == "r-t" ) { Wxxx = 5; Spcl = Spcl + 1 };
            if ( W1 == "rwT" ) { Wxxx = 6; Spcl = Spcl + 1 };
            if ( W1 == "rwt" ) { Wxxx = 7; Spcl = Spcl + 1 };

         # Rewrite to include logic to protect for when the link count is JAMMED against the permissions ('caused' when `ls` is forced to
         #   identify all the links to an entity, but abutt that numeric field RIGHT up against the permissions field without any
         #   intervening white-space.
         f3 = $3;                # Owner ID
         f4 = $4;                # Group ID
         f9 = substr($0,72);     # Get the entire part of the /path/to/the/file

         if ( Oxxx != 8 && Gxxx != 8 ) { Good = "True" }
                                  else { Good = "False" };
         if ( Good == "True" && Wxxx != 8 ) {
                # Using single quotes like this preserves funny characters and any spacing.
                print "echo '-------------------------------------------------------------------------------------------------'"  >> OutFile;
                print "echo \"" $0 "\""                                                                                           >> OutFile;
                print "chmod " Spcl Oxxx Gxxx Wxxx " " "\'" f9 "\'"                                                               >> OutFile;
                print "chown " f3 " " "\'" f9 "\'"                                                                                >> OutFile;
                print "chgrp " f4 " " "\'" f9 "\'"                                                                                >> OutFile;
                print " "                                                                                                         >> OutFile;
              }
         else {
                print "echo '[Strange Permissions]----------------------------------------------------------------------------'"  >> OutFile;
                print "echo \"" $0 "\""                                                                                           >> OutFile;
                print "echo \"O1(" O1 "), Oxxx(" Oxxx "), G1(" G1 "), Gxxx(" Gxxx "), W1(" W1 "), Wxxx(" Wxxx ").\""              >> OutFile;
                print " "                                                                                                         >> OutFile;
              }

             
       }

